
open Types
open Printf

let filename = ref None
let red = ref None
let black = ref None
let verbose = ref false
let dump = ref false
let speed = ref 1

let spec =
  [ "-v", Arg.Set verbose, "verbose";
    "-dump", Arg.Set dump, "dumps 10000 first rounds of game";
    "-world", Arg.String (fun s -> filename := Some s), "<world file>";
    "-red", Arg.String (fun s -> red := Some s), "<red code>";
    "-black", Arg.String (fun s -> black := Some s), "<black code>";
    "-speed", Arg.Int (fun s -> speed := s), "<speed>"]

let () = 
  Arg.parse spec (fun _ -> ()) "grs [options]";
  if !filename = None || !red = None || !black = None then begin
    Arg.usage spec "grs [options]"; 
    exit 1
  end

let rec input_nonspace_char ch =
  let c = input_char ch in
  if c=' ' then input_nonspace_char ch else c

let world, ant_hills, width, height = 
  let ch = open_in (match !filename with Some f -> f | None -> assert false) in
  let width = int_of_string (input_line ch) in
  let height = int_of_string (input_line ch) in
  let map = 
    Array.create_matrix width height 
      { rocky = false ; cell_ant = None ; ant_hill = None; food = 0 ;
	red_markers = [||] ; black_markers = [||] }
  in
  let ant_hills = ref [] in
  for y = 0 to height - 1 do
    for x = 0 to width - 1 do
      map.(x).(y) <-
      { rocky = false ; cell_ant = None ; ant_hill = None; food = 0 ;
	red_markers = [||] ; black_markers = [||] };      
      let c = input_nonspace_char ch in
      (match c with
	 | '#' -> map.(x).(y).rocky <- true 
	 | '.' -> ()
	 | '+' -> 
	     map.(x).(y).ant_hill <- Some Red;
	     ant_hills := (x,y,Red) :: !ant_hills
	 | '-' -> 
	     map.(x).(y).ant_hill <- Some Black;
	     ant_hills := (x,y,Black) :: !ant_hills
	 | '1'..'9' -> map.(x).(y).food <- int_of_char c - int_of_char '0'
	 | _ -> printf "Unknown character %c.@." c);
      if c <> '#' then begin
	map.(x).(y).red_markers <- Array.create 6 false;
	map.(x).(y).black_markers <- Array.create 6 false
      end
    done;
    let c = input_nonspace_char ch in
    assert (c = '\n')
  done;
  close_in ch;
  let ant_hills =
    List.sort
      (fun (x1,y1,_) (x2,y2,_) ->
	 match compare y1 y2 with
	   | 0 -> compare x1 x2
	   | n -> n)
      !ant_hills
  in
  map, ant_hills, width, height

let ants =
  let ants = 
    Array.init (List.length ant_hills)
      (fun i ->
	 { id = i ; color = Red ; state = 0 ; resting = 0 ;
	   position = (0,0);
	   direction = 0 ; has_food = false ; is_alive = true })
  in
  ignore 
    (List.fold_left
       (fun i (x,y,c) ->
	  world.(x).(y).cell_ant <- Some i;
	  let a = ants.(i) in
	  a.color <- c;
	  a.position <- (x,y);
	  i+1)
       0 ant_hills);
  ants

let read_brain filename =
  let ch = open_in (match !filename with Some f -> f | None -> assert false) in
  let buf = Lexing.from_channel ch in
  let instrs = Ant_parser.instrs_eof Ant_lexer.token buf in
  close_in ch;
  if !verbose then eprintf "%d instructions read@." (List.length instrs);
  Array.of_list instrs

let red_brain = read_brain red
let black_brain = read_brain black

let verbose = !verbose
let dump = !dump
let speed = !speed
