open Arg
open Inlining
open Flatten
open Resolve
open Printer

let source=ref None
let cible=ref None 
let verbose=ref false
let symbolic=ref false
	      
let anonfun s = 
  match !source with 
      Some _ -> raise (Arg.Bad s)
    | None -> source := Some s
	
let spec = [ "-o", String (fun s -> 
			     match !cible with 
				 Some _ -> raise (Arg.Bad ("-o "^s))
			       | None ->  cible:= Some s), 
	     "target file name (default stdout)";
	     "-v",  Set verbose,                 "verbose mode";
	     "-q",  Clear verbose,               "quiet mode";
	     "-s",  Set symbolic,  "dump intermediate code with named labels";
	     "--",  Rest anonfun, 
	     "who uses filenames starting with '-' anyway ?"]

let message = 
  "antmlc[.opt] [source] [options] compiles a fourmi program into an ant brain"
		
let main=
  let _=try
    Arg.parse spec anonfun message
  with Arg.Bad s ->
    Format.eprintf "I don't know what to do with '%s'\n" s;
    Arg.usage spec message;
    exit 1 in 
  let src,srcname=
    try
      (match !source with
	   None -> stdin,"<stdin>"
	 | Some s -> (open_in s,s))
    with Sys_error s ->
      Format.eprintf "System error: %s@." s; exit 2 in
  let ast = 
    try
      C_parser.prog C_lexer.token (Lexing.from_channel src)
    with
	C_lexer.Syntax_error n -> 
	  if n<0 then
	    Format.eprintf "Unfinished token at end of input %s @." srcname
	  else
	    Format.eprintf "Invalid token at line %d in %s @." n srcname;
	  exit 3
      | Parsing.Parse_error ->
	  let n= !C_lexer.line in
	    Format.eprintf "Syntax error at line %d in %s @." n srcname;
	    exit 4
  in
  let _ = close_in src in
  let monolithic_body = 
    try
      inline_prog ast
    with Scope_error s ->
      Format.eprintf "Scope error: %s@." s; exit 5
  in 
  let flat_body = flatten_prog monolithic_body in
  let out_formatter =
    match !cible with 
      None -> Format.std_formatter
    | Some s -> 
	try Format.formatter_of_out_channel (open_out s)     
	with Sys_error s ->
	  Format.eprintf "System error: %s@." s; exit 6 in
  let res = resolve_lbl flat_body in
  if !symbolic then
    print_flat_prog out_formatter flat_body
  else
    print_ant_brain out_formatter flat_body;
  flush_all ();
(*    close_out out_chan;*)
    exit 0 
    
