open Ast

exception Scope_error of string

let gen_lbl=
  let a=ref 0 in
    fun () -> incr a;("__"^string_of_int !a)
    
let rec has_duplicate=
  function 
      [] -> false
    | x :: q -> List.mem x q || has_duplicate q

let rec iter_labels_body f=
  List.iter (iter_labels_instr f)
and iter_labels_instr f=function
    Lbl s -> f s
  | If(_,b1,b2) -> iter_labels_body f b1;iter_labels_body f b2
  | While(_,b) | Repeat(_,b) | Seq b -> iter_labels_body f b
  | _ -> ()
      
let rec relocate_body f ret = List.map (relocate_instr f ret)
and relocate_instr f ret = function
    Lbl s -> Lbl(f s)
  | If(c,b1,b2) -> If(c,relocate_body f ret b1,relocate_body f ret b2)
  | While(c,b) -> While (c,relocate_body f ret b)
  | Repeat(c,b) -> Repeat (c,relocate_body f ret b)
  | Seq b -> Seq (relocate_body f ret b)
  | Pickup l -> Pickup (f l)
  | Move l -> Move (f l)
  | Goto l -> Goto (f l)
  | Call(p,args) -> assert false
  | Return -> Goto (ret) 
  | i -> i

let instantiate proc args=
  let label_env=Hashtbl.create 19 in
    (try
       List.iter2 (fun s s0-> 
		     if Hashtbl.mem label_env s then 
		       raise (Scope_error 
				("Duplicate parameter "^s^
				 " in definition "^proc.name));
		     Hashtbl.add label_env s s0) proc.args args with
	   Invalid_argument s -> 
	     raise (Scope_error ("Wrong number of arguments in call to "^s)));
    iter_labels_body (fun s -> 
			if Hashtbl.mem label_env s then 
			  raise (Scope_error ("Duplicate label "^s^
					      " in definition "^proc.name));
			Hashtbl.add label_env s (gen_lbl ())) proc.body;
    let ret=gen_lbl () in
    let rel s=
      try Hashtbl.find label_env s with
	  Not_found -> raise (Scope_error ("Unknown label "^s)) in
      (relocate_body rel ret proc.body)  @ [Lbl ret]
      
let rec inline_instr env i accu=
  match i with
      If(c,b1,b2) -> If(c,inline_body env b1 [],inline_body env b2 [])::accu
    | While(c,b) -> While (c,inline_body env b [])::accu
    | Repeat(c,b) -> Repeat (c,inline_body env b [])::accu
    | Seq b -> Seq (inline_body env b [])::accu
    | Call(p,args)-> 
	let gpdef=
	  try 
	    List.find (fun gp ->gp.name=p) env
	  with Not_found -> raise (Scope_error ("Unknown definition "^p)) in
	  (instantiate gpdef args)@accu
    | _ -> i::accu
and inline_body env=
  List.fold_right (inline_instr env)  

let inline_and_append accu p=
  {p with body=inline_body accu p.body []}::accu

let inline_prog prg=
  let inlined_env=List.fold_left inline_and_append [] prg.procs in
    inline_body inlined_env prg.main []
