open Ast

exception VarUndef of string

let gp = ref 0

let new_label = let r = ref 0 in fun () -> incr r; "_" ^ string_of_int !r

let ( genv : (string,Ast.expr) Hashtbl.t ) = Hashtbl.create 17 

let get_ident s = 
  try Hashtbl.find genv s
  with Not_found -> raise (VarUndef s)


let compile_dir = function
  | Here -> Instr.Here
  | Ahead -> Instr.Ahead
  | Lahead -> Instr.LeftAhead     
  | Rahead -> Instr.RightAhead

let compile_cond = function
  | Friend -> Instr.Friend
  | Foe -> Instr.Foe
  | Friendwithfood -> Instr.FriendWithFood
  | Foewithfood -> Instr.FoeWithFood    
  | Food -> Instr.Food           
  | Rock -> Instr.Rock
  | Marker i -> Instr.Marker i
  | Foemarker -> Instr.FoeMarker
  | Home -> Instr.Home
  | Foehome -> Instr.FoeHome

let compile_action error next pc = function
      Mark i -> pc , Instr.Mark(i,next) 
    | Unmark i -> pc , Instr.Unmark(i,next)
    | Pickup -> pc , Instr.PickUp(next,error)
    | Drop -> pc , Instr.Drop next
    | Lturn -> pc , Instr.Turn(Instr.Left,next)
    | Rturn -> pc, Instr.Turn(Instr.Right,next)
    | Move -> pc, Instr.Move(next,error)
    | Flip i -> pc , Instr.Flip(i,next,error)
    | Sense(c,d) -> pc , Instr.Sense(compile_dir d , next , error , compile_cond c)

let rec compile_expr pc error next = function
    Plus(e1,e2) -> 
      let l1 =  Label.create (new_label()) in
      let c1 = compile_expr pc l1 next e1 in
      let c2 = compile_expr l1 error next e2 in
	c1@c2
  | Whilenot(e1,e2) ->
      let pc' =  Label.create (new_label()) in
      let c1 = compile_expr pc pc' next e1 in
      let c2 = compile_expr pc' error pc e2 in
	c1@c2
  | Nopwhile e -> compile_expr pc pc next e
  | Seq(e1,e2) ->
      let pc' =  Label.create (new_label()) in
      let c1 = compile_expr pc error pc' e1 in
      let c2 = compile_expr pc' error next e2 in
	c1@c2
  | Ident s -> compile_expr pc error next (get_ident s)
  | Non e -> compile_expr pc next error e
  | Star e -> 
      let pc' =  Label.create (new_label()) in
      let c1 = compile_expr pc error pc' e in
      let c2 = compile_expr pc' next pc' e in
	c1@c2
  | Action a -> [compile_action error next pc a]




let compile_prg (macros,prog) ofile symb = 
  let f = open_out ofile in 
    Label.clear();
    List.iter (fun (x,e) -> Hashtbl.add genv x e) macros;
    let fmt = Format.formatter_of_out_channel f in
    let start = Label.create (new_label()) in
    let next =  Label.create (new_label()) in
    let code = compile_expr start next next prog in
      Emit.emit_code fmt next code symb;
      close_out f

    

