open Instr
open Format
open Label

let string_of_dir = function
    Here -> "Here"
  | Ahead -> "Ahead"          
  | LeftAhead -> "LeftAhead"     
  | RightAhead -> "RightAhead"

let string_of_cond = function
    Friend -> "Friend"
  | Foe -> "Foe"         
  | FriendWithFood -> "FriendWithFood"
  | FoeWithFood -> "FoeWithFood"
  | Food -> "Food"          
  | Rock -> "Rock"          
  | Marker i -> "Marker"^(string_of_int i)
  | FoeMarker -> "FoeMarker"      
  | Home -> "Home"          
  | FoeHome -> "FoeHome"  

let string_of_turn = function
    Left -> "Left"
  | Right -> "Right"

let label symb l = if symb then l.symbolic else (string_of_int l.address)

let concrete code next = 
  let pc = ref 0 in
    List.iter (fun (l,_) -> l.address<- !pc ; incr pc) code;
    next.address <- !pc

let emit fmt symb = function
    l , Sense(d,l1,l2,c) -> 
      fprintf fmt "Sense %s %s %s %s ; %s \n" 
      (string_of_dir d) (label symb l1) (label symb l2) (string_of_cond c) (label symb l) 
  | l , Mark(m,l') ->
      fprintf fmt "Mark %d %s ; %s \n"  m (label symb l') (label symb l)
  | l , Unmark(m,l') ->
      fprintf fmt "Unmark %d %s ; %s \n"  m (label symb l') (label symb l)
  | l , PickUp(l1,l2) ->
      fprintf fmt "PickUp %s %s ; %s \n" (label symb l1) (label symb l2) (label symb l) 
  | l , Drop l' ->
      fprintf fmt "Drop %s ; %s \n" (label symb l') (label symb l) 
  | l , Turn(t,l') -> 
      fprintf fmt "Turn %s %s ; %s \n" (string_of_turn t) (label symb l') (label symb l)
  | l , Move(l1,l2) -> 
      fprintf fmt "Move %s %s ; %s \n" (label symb l1) (label symb l2) (label symb l) 
  | l , Flip(i,l1,l2) -> 
      fprintf fmt "Flip %d %s %s ; %s \n" i (label symb l1) (label symb l2) (label symb l)

let emit_code fmt next code symb = 
  concrete code next;
  List.iter (emit fmt symb) code;
  Format.fprintf fmt "@?"
(*  Format.fprintf fmt "                   ; %s @?\n" (label symb next)*)
